window.Gleam.initSearch([{"doc":"bravo","title":"bravo","content":"# bravo\n### v1.0.0\n\n[![Package Version](https://img.shields.io/hexpm/v/bravo)](https://hex.pm/packages/bravo)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/bravo/)\n\n~~Comprehensive~~ Very much incomplete (for now) ETS bindings for Gleam.\n\nThis library is still in heavy development! Expect frequent breaking changes (this is indicated\nwhen the first number of the version increases).\n\nThis library is only compatible with the Erlang target!\n\n## What is an ETS?\n\nETS stands for Erlang Term Storage, and it is a data structure that stores collections of data\n(objects) that are addressable by keys.\n\n### Why should I use ETS over, say, a `Dict`?\n\nETS tables have the following properties that make them distinct from `Dict`s:\n- ETS tables are not garbage collected, allowing them to be accessed from anywhere in a process\n  without needing to pass return values around. This does come at the cost of forcing the user\n  to manually delete the table once they are done with it.\n- ETS tables offer a wide array of customization options that `Dict`s do not, such as the options\n  to be ordered, allow multiple objects per key, which element of the object acts as the key,\n  and controlling read/write access from external processes.\n- ETS tables offer increased functionality compared to `Dict`s, natively allowing functions such as \n  complex searching, reading/writing the ETS to a file, and having much more complex entries.\n- ETS tables work well with concurrency.\n\n### Why should I use Bravo over other libraries such as Carpenter?\n\nA difficulty of implementing ETS tables in Gleam is that Gleam is a statically-typed language and\nETS tables are very much designed for the dynamically-typed Erlang. Carpenter ensures type safety\nby restricting objects to simple key-value pairs, much like a `Dict`.\n\nBravo, however, sacrifices type guarantees to empower the ETS table. ETS tables do not store type\ninformation, so the burden of interpreting the binary data stored in the ETS into a usable Gleam\ntype is placed on the user. Fortunately, the `gleam/dynamic` library provides powerful tools to\nwork around this. As long as you know what each object&#39;s type signature is, you should be fine.\n\n## Installation\nBravo depends on the `gleam_erlang` package, but otherwise does not have any other dependency.\n\n```sh\ngleam add bravo gleam_erlang\n```\n\n## Usage\n```gleam\nimport bravo/etc\nimport bravo/object\nimport bravo/uset\nimport gleam/dynamic\nimport gleam/io\nimport gleam/option.{Some}\n\npub fn main() {\n  // Create a new ETS table. There are multiple options, but here we are using\n  // a USet (an alias for &quot;set&quot; in Erlang terms)\n  let assert Ok(table) = uset.new(&quot;MyTable&quot;, 1, etc.Public)\n\n  // We can then insert tuples into this table\n  uset.insert(table, [#(&quot;Hello&quot;, &quot;world!&quot;)])\n\n  // Then we can lookup the object from the table\n  let assert Some(object) = uset.lookup(table, &quot;Hello&quot;)\n  let assert Ok(tuple) =\n    object\n    |&gt; object.extract\n    |&gt; dynamic.tuple2(dynamic.string, dynamic.string)\n  io.print(tuple.0 &lt;&gt; &quot;, &quot; &lt;&gt; tuple.1) // &quot;Hello, world!&quot;\n\n  // ETS tables have static lifetimes,\n  // so don&#39;t forget to delete them when you&#39;re done!\n  uset.delete(table)\n}\n```\n\nFurther documentation can be found at &lt;https://hexdocs.pm/bravo&gt;.\n","url":"index.html"},{"doc":"bravo/etc","title":"Access","content":"pub type Access {\n  Public\n  Protected\n  Private\n}\n Access specifiers for ETS tables. Affects how other processes can interact with the table.\n\nPublic\n Any process can read or write to the `USet`.\n\nProtected\n Any process can read the `USet`. Only the owner process can write to it.\n\nPrivate\n Only the parent process can read or write to the `USet`.\n\nSynonyms:\nbravo/etc.Access\nbravo/etc Access","url":"bravo/etc.html#Access"},{"doc":"bravo/etc","title":"Term","content":"pub type Term\n Represents an Erlang `term()`, which can represent any data type.\n\n\nSynonyms:\nbravo/etc.Term\nbravo/etc Term","url":"bravo/etc.html#Term"},{"doc":"bravo/etc","title":"bravo/etc","content":" This module provides types or functions that don&#39;t fit anywhere else.","url":"bravo/etc.html"},{"doc":"bravo/error","title":"ErlangError","content":"pub type ErlangError {\n  Badarg\n  Badarith\n  Badmatch(Term)\n  FunctionClause\n  CaseClause(Term)\n  IfClause\n  TryClause(Term)\n  Undef\n  Badfun(Term)\n  Badarity(#(Term, Term))\n  TimeoutValue\n  Noproc\n  Noconnection\n  Nocatch(Term)\n  SystemLimit\n}\n Represents any of Erlang&#39;s runtime errors. These errors are often vague and unclear.\n\nBadarg\n\n\nBadarith\n\n\nBadmatch(Term)\n\n\nFunctionClause\n\n\nCaseClause(Term)\n\n\nIfClause\n\n\nTryClause(Term)\n\n\nUndef\n\n\nBadfun(Term)\n\n\nBadarity(#(Term, Term))\n\n\nTimeoutValue\n\n\nNoproc\n\n\nNoconnection\n\n\nNocatch(Term)\n\n\nSystemLimit\n\n\nSynonyms:\nbravo/error.ErlangError\nbravo/error ErlangError","url":"bravo/error.html#ErlangError"},{"doc":"bravo/error","title":"bravo/error","content":" This module provides error types specific to Bravo. Currently, this only includes `ErlangError`","url":"bravo/error.html"},{"doc":"bravo/object","title":"Object","content":"pub opaque type Object(a)\n Represents an object in an ETS table. Functionally, this is simply a wrapper around arbitrary tuples.\n\n\nSynonyms:\nbravo/object.Object\nbravo/object Object","url":"bravo/object.html#Object"},{"doc":"bravo/object","title":"element","content":"pub fn element(object: Object(a), index: Int) -&gt; Option(Dynamic)\n Gets the nth element of an `Object` (currently 1-indexed but this may change). Fails if the index is out of range.\n\nSynonyms:\nbravo/object.element\nbravo/object element","url":"bravo/object.html#element"},{"doc":"bravo/object","title":"extract","content":"pub fn extract(object: Object(a)) -&gt; Dynamic\n Creates a `Dynamic` from the data stored inside an `Object`. This should always be a tuple.\n\nSynonyms:\nbravo/object.extract\nbravo/object extract","url":"bravo/object.html#extract"},{"doc":"bravo/object","title":"new","content":"pub fn new(tuple: a) -&gt; Object(a)\n Creates a new `Object` from a tuple. Passing in a non-tuple may result in unexpected behavior.\n\nSynonyms:\nbravo/object.new\nbravo/object new","url":"bravo/object.html#new"},{"doc":"bravo/object","title":"size","content":"pub fn size(object: Object(a)) -&gt; Int\n Get the number of elements in an `Object`.\n\nSynonyms:\nbravo/object.size\nbravo/object size","url":"bravo/object.html#size"},{"doc":"bravo/object","title":"bravo/object","content":" This module provides the `Object` type and functions that use it. `Object`s are essential to accessing any ETS table.","url":"bravo/object.html"},{"doc":"bravo/bag","title":"Bag","content":"pub opaque type Bag\n A bag table. Keys may occur multiple times per table, but objects cannot be copied verbatim.\n\n\nSynonyms:\nbravo/bag.Bag\nbravo/bag Bag","url":"bravo/bag.html#Bag"},{"doc":"bravo/bag","title":"delete","content":"pub fn delete(bag: Bag) -&gt; Bool\n Deletes a `Bag`.\n\n Table lifetime is static, and memory is only freed when the owner process is killed! Don&#39;t forget to call this function!\n\nSynonyms:\nbravo/bag.delete\nbravo/bag delete","url":"bravo/bag.html#delete"},{"doc":"bravo/bag","title":"insert","content":"pub fn insert(bag: Bag, objects: List(a)) -&gt; Bool\n Inserts a list of tuples into a `Bag`.\n\n Returns a `Bool` representing if the inserting succeeded. \n - If `True`, all objects in the list were inserted.\n - If `False`, _none_ of the objects in the list were inserted. This may occur if the size of the tuple is less than the `Bag`&#39;s size.\n\n If an `Object` with the same key already exists, then the old `Object` will be overwritten with the new one.\n\nSynonyms:\nbravo/bag.insert\nbravo/bag insert","url":"bravo/bag.html#insert"},{"doc":"bravo/bag","title":"insert_obj","content":"pub fn insert_obj(bag: Bag, objects: List(Object(a))) -&gt; Bool\n Inserts a list of `Object`s into a `Bag`. It is recommended to use `insert` instead when possible, as this uses that function under the hood.\n\n Returns a `Bool` representing if the inserting succeeded. \n - If `True`, all objects in the list were inserted.\n - If `False`, _none_ of the objects in the list were inserted. This may occur if the size of the tuple is less than the `Bag`&#39;s size.\n\n If an `Object` with the same key already exists, then the old `Object` will be overwritten with the new one.\n\nSynonyms:\nbravo/bag.insert_obj\nbravo/bag insert_obj","url":"bravo/bag.html#insert_obj"},{"doc":"bravo/bag","title":"lookup","content":"pub fn lookup(bag: Bag, key: a) -&gt; List(Object(Dynamic))\n Gets a list of `Object`s from a `Bag`.\n\n - If the list isn&#39;t empty, then at least one object was found. ETS tables do not store types, so you must decode a `Dynamic` inside the `Object`.\n - If the list is empty, then the `Bag` did not contain any `Object` with the specified `key`.\nSynonyms:\nbravo/bag.lookup\nbravo/bag lookup","url":"bravo/bag.html#lookup"},{"doc":"bravo/bag","title":"new","content":"pub fn new(\n  name: String,\n  keypos: Int,\n  access: Access,\n) -&gt; Result(Bag, Option(ErlangError))\n Creates a new ETS table configured as a bag: keys may only occur multiple times per table, but objects cannot be copied verbatim.\n\n `name`: An atom representing the name of the `Bag`. There may only be one ETS table associated with an atom.\n `keypos`: The index (1-indexed) that represents the key position of the object. This function fails if this is less than 1.\n `access`: Determines how visible the table is to other processes.\n - `Public`: Any process can read or write to the `Bag`.\n - `Protected`: Any process can read the `Bag`. Only the owner process can write to it.\n - `Private`: Only the parent process can read or write to the `Bag`.\n\n Returns a result of the created `Bag`, which can be used by other functions in this module.\n If this function errors with `None`, then you likely put in an illegal `keypos` value.\n Otherwise, something went wrong in the FFI layer and an error occured in Erlang land.\n\nSynonyms:\nbravo/bag.new\nbravo/bag new","url":"bravo/bag.html#new"},{"doc":"bravo/bag","title":"bravo/bag","content":" This module provides functions to work with `Bag`s","url":"bravo/bag.html"},{"doc":"bravo/dbag","title":"DBag","content":"pub opaque type DBag\n A duplicate bag etc. Keys may occur multiple times per table, and verbatim copies of an object can be stored.\n\n\nSynonyms:\nbravo/dbag.DBag\nbravo/dbag DBag","url":"bravo/dbag.html#DBag"},{"doc":"bravo/dbag","title":"delete","content":"pub fn delete(dbag: DBag) -&gt; Bool\n Deletes a `DBag`.\n\n Table lifetime is static, and memory is only freed when the owner process is killed! Don&#39;t forget to call this function!\n\nSynonyms:\nbravo/dbag.delete\nbravo/dbag delete","url":"bravo/dbag.html#delete"},{"doc":"bravo/dbag","title":"insert","content":"pub fn insert(dbag: DBag, objects: List(a)) -&gt; Bool\n Inserts a list of tuples into a `DBag`.\n\n Returns a `Bool` representing if the inserting succeeded. \n - If `True`, all objects in the list were inserted.\n - If `False`, _none_ of the objects in the list were inserted. This may occur if the size of the tuple is less than the `DBag`&#39;s size.\n\n If an `Object` with the same key already exists, then the old `Object` will be overwritten with the new one.\n\nSynonyms:\nbravo/dbag.insert\nbravo/dbag insert","url":"bravo/dbag.html#insert"},{"doc":"bravo/dbag","title":"insert_obj","content":"pub fn insert_obj(dbag: DBag, objects: List(Object(a))) -&gt; Bool\n Inserts a list of `Object`s into a `DBag`. It is recommended to use `insert` instead when possible, as this uses that function under the hood.\n\n Returns a `Bool` representing if the inserting succeeded. \n - If `True`, all objects in the list were inserted.\n - If `False`, _none_ of the objects in the list were inserted. This may occur if the size of the tuple is less than the `DBag`&#39;s size.\n\n If an `Object` with the same key already exists, then the old `Object` will be overwritten with the new one.\n\nSynonyms:\nbravo/dbag.insert_obj\nbravo/dbag insert_obj","url":"bravo/dbag.html#insert_obj"},{"doc":"bravo/dbag","title":"lookup","content":"pub fn lookup(dbag: DBag, key: a) -&gt; List(Object(Dynamic))\n Gets an `Object` from a `DBag`.\n\n Returns an `Option` containing the object, if it exists.\n - If `Some`, then the object was found. ETS tables do not store types, so you must decode a `Dynamic` inside the `Object`.\n - If `None`, then the `DBag` did not contain any `Object` with the specified `key`.\nSynonyms:\nbravo/dbag.lookup\nbravo/dbag lookup","url":"bravo/dbag.html#lookup"},{"doc":"bravo/dbag","title":"new","content":"pub fn new(\n  name: String,\n  keypos: Int,\n  access: Access,\n) -&gt; Result(DBag, Option(ErlangError))\n Creates a new ETS table configured as a duplicate bag: keys may occur multiple times per table, and verbatim copies of an object can be stored.\n\n `name`: An atom representing the name of the `DBag`. There may only be one ETS table associated with an atom.\n `keypos`: The index (1-indexed) that represents the key position of the object. This function fails if this is less than 1.\n `access`: Determines how visible the table is to other processes.\n - `Public`: Any process can read or write to the `DBag`.\n - `Protected`: Any process can read the `DBag`. Only the owner process can write to it.\n - `Private`: Only the parent process can read or write to the `DBag`.\n\n Returns a result of the created `DBag`, which can be used by other functions in this module.\n If this function errors with `None`, then you likely put in an illegal `keypos` value.\n Otherwise, something went wrong in the FFI layer and an error occured in Erlang land.\n\nSynonyms:\nbravo/dbag.new\nbravo/dbag new","url":"bravo/dbag.html#new"},{"doc":"bravo/dbag","title":"bravo/dbag","content":" This module provides functions to work with `DBag`s","url":"bravo/dbag.html"},{"doc":"bravo/oset","title":"OSet","content":"pub opaque type OSet\n An ordered set. Keys may only occur once per table, and objects are ordered (this comes at a performance cost).\n\n In order for a lookup match to occur, entries must _coerce into the same value_. Two values may match even if they have different types.\n\n\nSynonyms:\nbravo/oset.OSet\nbravo/oset OSet","url":"bravo/oset.html#OSet"},{"doc":"bravo/oset","title":"delete","content":"pub fn delete(oset: OSet) -&gt; Bool\n Deletes a `OSet`.\n\n Table lifetime is static, and memory is only freed when the owner process is killed! Don&#39;t forget to call this function!\n\nSynonyms:\nbravo/oset.delete\nbravo/oset delete","url":"bravo/oset.html#delete"},{"doc":"bravo/oset","title":"insert","content":"pub fn insert(oset: OSet, objects: List(a)) -&gt; Bool\n Inserts a list of tuples into a `OSet`.\n\n Returns a `Bool` representing if the inserting succeeded. \n - If `True`, all objects in the list were inserted.\n - If `False`, _none_ of the objects in the list were inserted. This may occur if the size of the tuple is less than the `OSet`&#39;s size.\n\n If an `Object` with the same key already exists, then the old `Object` will be overwritten with the new one.\n\nSynonyms:\nbravo/oset.insert\nbravo/oset insert","url":"bravo/oset.html#insert"},{"doc":"bravo/oset","title":"insert_obj","content":"pub fn insert_obj(oset: OSet, objects: List(Object(a))) -&gt; Bool\n Inserts a list of `Object`s into a `OSet`. It is recommended to use `insert` instead when possible, as this uses that function under the hood.\n\n Returns a `Bool` representing if the inserting succeeded. \n - If `True`, all objects in the list were inserted.\n - If `False`, _none_ of the objects in the list were inserted. This may occur if the size of the tuple is less than the `OSet`&#39;s size.\n\n If an `Object` with the same key already exists, then the old `Object` will be overwritten with the new one.\n\nSynonyms:\nbravo/oset.insert_obj\nbravo/oset insert_obj","url":"bravo/oset.html#insert_obj"},{"doc":"bravo/oset","title":"lookup","content":"pub fn lookup(oset: OSet, key: a) -&gt; Option(Object(Dynamic))\n Gets an `Object` from a `OSet`.\n\n Returns an `Option` containing the object, if it exists.\n - If `Some`, then the object was found. ETS tables do not store types, so you must decode a `Dynamic` inside the `Object`.\n - If `None`, then the `OSet` did not contain any `Object` with the specified `key`.\n\nSynonyms:\nbravo/oset.lookup\nbravo/oset lookup","url":"bravo/oset.html#lookup"},{"doc":"bravo/oset","title":"new","content":"pub fn new(\n  name: String,\n  keypos: Int,\n  access: Access,\n) -&gt; Result(OSet, Option(ErlangError))\n Creates a new ETS table configured as an ordered set: keys may only occur once per table, and objects are ordered (this comes at a performance cost).\n\n `name`: An atom representing the name of the `OSet`. There may only be one ETS table associated with an atom.\n `keypos`: The index (1-indexed) that represents the key position of the object. This function fails if this is less than 1.\n `access`: Determines how visible the table is to other processes.\n - `Public`: Any process can read or write to the `OSet`.\n - `Protected`: Any process can read the `OSet`. Only the owner process can write to it.\n - `Private`: Only the parent process can read or write to the `OSet`.\n\n Returns a result of the created `OSet`, which can be used by other functions in this module.\n If this function errors with `None`, then you likely put in an illegal `keypos` value.\n Otherwise, something went wrong in the FFI layer and an error occured in Erlang land.\n\nSynonyms:\nbravo/oset.new\nbravo/oset new","url":"bravo/oset.html#new"},{"doc":"bravo/oset","title":"bravo/oset","content":" This module provides functions to work with `OSet`s","url":"bravo/oset.html"},{"doc":"bravo/uset","title":"USet","content":"pub opaque type USet\n An unordered set. Keys may only occur once per table, and objects are unordered.\n\n In order for a lookup match to occur, entries must have the same value _and type_.\n\n\nSynonyms:\nbravo/uset.USet\nbravo/uset USet","url":"bravo/uset.html#USet"},{"doc":"bravo/uset","title":"delete","content":"pub fn delete(uset: USet) -&gt; Bool\n Deletes a `USet`.\n\n Table lifetime is static, and memory is only freed when the owner process is killed! Don&#39;t forget to call this function!\n\nSynonyms:\nbravo/uset.delete\nbravo/uset delete","url":"bravo/uset.html#delete"},{"doc":"bravo/uset","title":"insert","content":"pub fn insert(uset: USet, objects: List(a)) -&gt; Bool\n Inserts a list of tuples into a `USet`.\n\n Returns a `Bool` representing if the inserting succeeded. \n - If `True`, all objects in the list were inserted.\n - If `False`, _none_ of the objects in the list were inserted. This may occur if the size of the tuple is less than the `USet`&#39;s size.\n\n If an `Object` with the same key already exists, then the old `Object` will be overwritten with the new one.\n\nSynonyms:\nbravo/uset.insert\nbravo/uset insert","url":"bravo/uset.html#insert"},{"doc":"bravo/uset","title":"insert_obj","content":"pub fn insert_obj(uset: USet, objects: List(Object(a))) -&gt; Bool\n Inserts a list of `Object`s into a `USet`. It is recommended to use `insert` instead when possible, as this uses that function under the hood.\n\n Returns a `Bool` representing if the inserting succeeded. \n - If `True`, all objects in the list were inserted.\n - If `False`, _none_ of the objects in the list were inserted. This may occur if the size of the tuple is less than the `USet`&#39;s size.\n\n If an `Object` with the same key already exists, then the old `Object` will be overwritten with the new one.\n\nSynonyms:\nbravo/uset.insert_obj\nbravo/uset insert_obj","url":"bravo/uset.html#insert_obj"},{"doc":"bravo/uset","title":"lookup","content":"pub fn lookup(uset: USet, key: a) -&gt; Option(Object(Dynamic))\n Gets an `Object` from a `USet`.\n\n Returns an `Option` containing the object, if it exists.\n - If `Some`, then the object was found. ETS tables do not store types, so you must decode a `Dynamic` inside the `Object`.\n - If `None`, then the `USet` did not contain any `Object` with the specified `key`.\n\nSynonyms:\nbravo/uset.lookup\nbravo/uset lookup","url":"bravo/uset.html#lookup"},{"doc":"bravo/uset","title":"new","content":"pub fn new(\n  name: String,\n  keypos: Int,\n  access: Access,\n) -&gt; Result(USet, Option(ErlangError))\n Creates a new ETS table configured as a set: keys may only occur once per table, and objects are unordered.\n\n `name`: An atom representing the name of the `USet`. There may only be one ETS table associated with an atom.\n `keypos`: The index (1-indexed) that represents the key position of the object. This function fails if this is less than 1.\n `access`: Determines how visible the table is to other processes.\n - `Public`: Any process can read or write to the `USet`.\n - `Protected`: Any process can read the `USet`. Only the owner process can write to it.\n - `Private`: Only the parent process can read or write to the `USet`.\n\n Returns a result of the created `USet`, which can be used by other functions in this module.\n If this function errors with `None`, then you likely put in an illegal `keypos` value.\n Otherwise, something went wrong in the FFI layer and an error occured in Erlang land.\n\nSynonyms:\nbravo/uset.new\nbravo/uset new","url":"bravo/uset.html#new"},{"doc":"bravo/uset","title":"bravo/uset","content":" This module provides functions to work with `USet`s","url":"bravo/uset.html"}]);